# $Id$

package XML::Atom::Server::Lite::Entry;
use strict;
use XML::Atom::Server::Lite qw/qquote encode_xml/;

sub new {
    my $class = shift;
    my $self = bless {}, $class;
    $self->{id} = '';
    $self->{logo} = '';
    $self->{icon} = '';
    $self->{title} = '';
    $self->{link} = [];
    $self->{subject} = [];
    $self->{author} = [];
    $self->{content} = {};
    $self->{category} = {};
    $self->{published} = '';
    $self->{updated} = '';
    $self;
}

sub title {
    my $self = shift;
    if (@_) { $self->{title} = $_[0] } else { $self->{title}||'' };
}

sub id {
    my $self = shift;
    if (@_) { $self->{id} = $_[0] } else { $self->{id}||'' };
}

sub logo {
    my $self = shift;
    if (@_) { $self->{logo} = $_[0] } else { $self->{logo}||'' };
}

sub icon {
    logo(@_);
}

sub author {
    my $self = shift;
    if (@_) { $self->{author} = \@_ } else { $self->{author}||() };
}

sub published {
    my $self = shift;
    if (@_) { $self->{published} = \@_ } else { $self->{published}||() };
}

sub updated {
    my $self = shift;
    if (@_) { $self->{updated} = \@_ } else { $self->{updated}||() };
}

sub link {
    my $self = shift;
    if (@_) {
        if (ref($_[0]) eq 'ARRAY') {
            $self->{link} = $_[0];
        } else {
            push @{$self->{link}}, $_[0];
        }
    } else { $self->{link}||(); };
}

sub subject {
    my $self = shift;
    if (@_) {
        if (ref($_[0]) eq 'ARRAY') {
            $self->{subject} = $_[0];
        } else {
            push @{$self->{subject}}, $_[0];
        }
    } else { $self->{subject}||(); };
}

sub category {
    my $self = shift;
    if (@_) { $self->{category} = $_[0] } else { $self->{category}||{} };
}

sub content {
    my $self = shift;
    if (@_) { $self->{content} = $_[0] } else { $self->{content}||{} };
}

sub to_xml {
    my $self = shift;
    my $xml = "";
    $xml .= " <entry>\n";
    for my $link (@{$self->{link}}) {
        $xml .= "  <link"
             .  " rel=".XML::Atom::Server::Lite::qquote($link->{rel})
             .  " title=".XML::Atom::Server::Lite::qquote($link->{title})
             .  " type=".XML::Atom::Server::Lite::qquote($link->{type})
             .  " href=".XML::Atom::Server::Lite::qquote($link->{href})."/>\n";
    }
    $xml .= "  <title>"
           . XML::Atom::Server::Lite::encode_xml($self->{title})
           . "</title>\n" if $self->{title};
    $xml .= "  <icon>"
           . XML::Atom::Server::Lite::encode_xml($self->{icon})
           . "</icon>\n" if $self->{icon};
    $xml .= "  <logo>"
           . XML::Atom::Server::Lite::encode_xml($self->{logo})
           . "</logo>\n" if $self->{logo};
    $xml .= "  <content"
         .  " mode=".XML::Atom::Server::Lite::qquote($self->{content}->{mode})
         .  " type=".XML::Atom::Server::Lite::qquote($self->{content}->{type}).">"
         .  "<div xmlns=\"http://www.w3.org/1999/xhtml\">"
         .  XML::Atom::Server::Lite::encode_xml($self->{content}->{body})
         .  "</div></content>\n" if $self->{content};
    $xml .= "  <category"
            ." term=".XML::Atom::Server::Lite::qquote($self->{category}->{term})
            ." label=".XML::Atom::Server::Lite::qquote($self->{category}->{label})."/>\n" if $self->{category};
    for my $subject (@{$self->{subject}}) {
        $xml .= "  <dc:subject>"
               . XML::Atom::Server::Lite::encode_xml($subject)
               . "</dc:subject>\n";
    }
    $xml .= "  <published>"
           . XML::Atom::Server::Lite::encode_xml($self->{published})
           . "</published>\n" if $self->{published};
    $xml .= "  <updated>"
           . XML::Atom::Server::Lite::encode_xml($self->{updated})
           . "</updated>\n" if $self->{updated};
    $xml .= " </entry>\n";
}

1;
