# sb::Admin::Home
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==

# ======================================
package sb::Admin::Home;
use strict;
use sb::Admin::Entry ();
use base qw( sb::Admin::Entry );
use vars qw( $VERSION );
$VERSION = '0.09';
# 0.09 [2010/06/14] modified _display_plugin to use dfn for plugin type
# 0.08 [2009/03/19] removed br tags from the strings
# 0.07 [2009/03/07] added _get_available_languages / modified _update to consider changing language
# 0.06 [2009/02/09] modified title_name to make it clearer / 
# 0.05 [2007/12/02] modified _display_plugin to display plugin correctly
# 0.04 [2007/11/05] added title_name
# 0.03 [2007/09/26] enabled $self->conf->save
# 0.02 [2007/09/09] ported from Serene Bach 2.* (sb::Admin::Status)
# 0.01 [2005/08/06] chnaged _clip_for_entry/comment/trackback to display list correctly
# 0.00 [2005/02/02] generated
# --------------------------------------
# // defines
sub TEMPLATE      (){ 'home.html' };
sub STATUS_NUM    (){ 10 };
sub DEFAULT_TYPE  (){ 'status' };
sub PLUGIN_LINK   (){ '<a href="%s">%s</a>' };
sub PLUGIN_NONAME (){ '-' };
sub PLUGIN_DETAIL (){ '<dfn title="%s">%s</dfn>' };
sub CONFIG_LEVEL  (){ 1 }
my %mConfigType = (
	'status'   => {'submit'=>0,},
	'display'  => {'submit'=>1,},
	'basic'    => {'submit'=>1,},
	'plugin'   => {'submit'=>0,},
);
my %mConfigItems = (
	'disp' => ['entry_disp','newent_disp','com_disp','tb_disp','aws_disp',],
	'sort' => ['page_disp','search_disp','entry_sort','archive_sort','com_sort','tb_sort',],
	'base' => ['srv_base','dir_base','dir_log','dir_img',],
);
# --------------------------------------
# // override base class
sub title_name
{
	my $self = shift;
	my $title = $self->SUPER::title_name();
	return $title if ($title);
	$title = sb->string(' &gt; Home');
	$title .= sb->string(
		  ($self->{'type'} eq 'display') ? ' &gt; Weglog settings'
		: ($self->{'type'} eq 'basic')   ? ' &gt; Server settings'
		: ($self->{'type'} eq 'plugin')  ? ' &gt; Plugins'
		:                                  '');
	return $title;
}
# --------------------------------------
# // public functions
sub callback
{
	my $self = shift;
	$self->{'blog'} = sb::Data->load('Weblog','id'=>0) if ( !defined($self->{'blog'}) );
	$self->{'type'} = $self->cgi->value('__type');
	$self->{'type'} = DEFAULT_TYPE if (!$mConfigType{$self->{'type'}});
	return ($self->regi)
		? $self->_update(@_)
		: $self->_display(@_);
}
# --------------------------------------
# // private functions
sub _update
{
	my $self = shift;
	if (!$self->has_permission('level'=>CONFIG_LEVEL))
	{
		$self->{'type'} = DEFAULT_TYPE;
		return $self->_display('message'=>sb->string('Not authorized.'));
	}
	my $reload = undef;
	SWITCH_TYPE: {
		$_ = $self->{'type'};
		/^status$/ && do
		{ # it does not normally happen.
			return $self->_display('message'=>'');
		};
		/^display$/ && do
		{
			$reload = $self->_update_appearance();
			last SWITCH_TYPE;
		};
		/^basic$/ && do
		{
			$reload = $self->_update_basic();
			last SWITCH_TYPE;
		};
		/^plugin$/ && do
		{
			$reload = $self->_update_plugin();
			last SWITCH_TYPE;
		};
	} # end of SWITCH_TYPE
	sb::Data->update($self->{'blog'});
	$self->conf->save($self->conf->config_file());
	if ( !$reload )
	{
		return $self->_display('message'=>$self->_get_message());
	}
	else
	{ # changed the plugin settings, so we should redirect to apply new settings.
		$self->cgi->head('location'=>$self->script_path . '?__mode=home&__type=' . $reload . '&change=on');
		return;
	}
}
sub _display
{
	my $self = shift;
	my %param = (
		'message' => undef,
		@_
	);
	my $temp = sb::Template->new($self->load_template('file'=>TEMPLATE));
	SWITCH_TYPE:
	{
		$_ = $self->{'type'};
		/^status$/ && do
		{
			$self->_display_status($temp);
			last SWITCH_TYPE;
		};
		/^display$/ && do
		{
			$self->_display_appearance($temp);
			last SWITCH_TYPE;
		};
		/^basic$/ && do
		{
			$self->_display_basic($temp);
			last SWITCH_TYPE;
		};
		/^plugin$/ && do
		{
			$self->_display_plugin($temp);
			last SWITCH_TYPE;
		};
	} # end of SWITCH_TYPE
	$param{'message'} = $self->_get_message() if ($self->cgi->value('change') eq 'on');
	$temp->num(0);
	$temp->tag('sb_selected_type' => $self->{'type'});
	$temp->tag('sb_home_menu_' . $self->{'type'} => ' class="current"');
	$temp->block('sb_home_typelist' => int($self->has_permission('level'=>CONFIG_LEVEL)) );
	$temp->block('sb_home_submit' => $mConfigType{$self->{'type'}}->{'submit'});
	$temp->block('sb_home_' . $self->{'type'} => 1);
	$self->common_parts($temp);
	$self->set_message($temp,$param{'message'});
	return $self->set_main($temp->output)
}
# --------------------------------------
# // private functions - update
sub _update_appearance
{
	my $self = shift;
	$self->{'blog'}->title(sb::Text->new($self->cgi->value('blog_title'))->entitized());
	$self->{'blog'}->text(sb::Text->new($self->cgi->value('blog_desc'))->entitized());
	foreach my $key ( @{$mConfigItems{'disp'}} )
	{
		my $value = int($self->cgi->value($key));
		$value = 1 if ($value <= 0);
		$self->conf->value('conf_' . $key => $value);
	}
	foreach my $key ( @{$mConfigItems{'sort'}} )
	{
		$self->conf->value('conf_' . $key => int($self->cgi->value($key)));
	}
	return;
}
sub _update_basic
{
	my $self = shift;
	$self->conf->value('conf_entry_archive'=>$self->cgi->value('entry_archive'));
	$self->conf->value('conf_imagename'=>int($self->cgi->value('imagename')));
	if ($self->cgi->value('thumbcheck') ne '')
	{
		$self->conf->value('conf_thumbcheck'=>int($self->cgi->value('thumbcheck')));
	}
	if ($self->cgi->value('thumbsize') ne '')
	{
		my $value = int($self->cgi->value('thumbsize'));
		$value = 0 if ($value <= 0);
		$self->conf->value('conf_thumbsize'=>$value);
	}
	foreach my $key ( @{$mConfigItems{'base'}} )
	{
		my $value = $self->cgi->value($key);
		$value .= '/' if ($value !~ /\/$/); # check the directory name
		$self->conf->value('conf_' . $key => $value);
	}
	$self->conf->value('conf_timezone' => $self->cgi->value('tz_hour') . $self->cgi->value('tz_min'));
	$self->{'blog'}->stype($self->cgi->value('sendmail'));
	$self->{'blog'}->smtp($self->cgi->value('smtp_address'));
	$self->{'blog'}->psrv($self->cgi->value('popserver'));
	$self->{'blog'}->pacc($self->cgi->value('popacount'));
	$self->{'blog'}->ppass($self->cgi->value('poppassword'));
	$self->{'blog'}->psubj($self->cgi->value('popsubject'));
	if (  $self->cgi->value('lang') ne '' 
	  and $self->cgi->value('lang') ne $self->lang->language )
	{
		my @langs = $self->_get_available_languages();
		foreach my $lang ( @langs )
		{
			next if ( $lang ne $self->cgi->value('lang') );
			$self->conf->value('conf_lang' => $lang);
			return 'basic';
		}
	}
	return;
}
sub _update_plugin
{
	my $self = shift;
	my @plugins = split("\0",$self->cgi->value('use'));
	my $setting = '';
	foreach my $plugin (@plugins)
	{
		$plugin =~ s/\[/&#91;/g;
		$plugin =~ s/\]/&#93;/g;
		$setting .= '[' . $plugin . ']';
	}
	$self->{'blog'}->plugin($setting);
	return 'plugin';
}
# --------------------------------------
# // private functions - display
sub _display_status
{
	my $self = shift;
	my $temp = shift;
	$temp->num(0);
	$temp->tag('sb_status_num'=>STATUS_NUM);
	$temp->tag('sb_blog_name'=>$self->{'blog'}->title);
	{ # --- entries in the weblog
		$temp->num(0);
		$temp->tag('sb_entall_close'=>sb::Data->count('Entry','cond'=>{'stat'=>0}));
		my @entry = sb::Data->load('Entry',
			'sort'  => 'date',
			'order' => 1,
			'num'   => STATUS_NUM,
		);
		$temp->tag('sb_entall_num'=>sb::Data->matched);
		$self->listmain(
			'template' => $temp,
			'block'    => 'sb_status_entall_list',
			'objects'  => \@entry,
			'tags'     => {
				'sb_entall_subj' => sub { (shift)->display_entry_title(@_) },
				'sb_entall_date' => 'date',
				'sb_entall_stat' => sub { (shift)->display_entry_status(@_) },
			},
		);
	}
	{ # --- entries of the current user
		$temp->num(0);
		$temp->tag('sb_entusr_close'=>sb::Data->count('Entry','cond'=>{'auth'=>$self->user->id,'stat'=>0}));
		my @entry = sb::Data->load('Entry',
			'sort'  => 'date',
			'order' => 1,
			'cond'  => {'auth'=>$self->{'user'}->id},
			'num'   => STATUS_NUM,
		);
		$temp->tag('sb_entusr_num'=>sb::Data->matched);
		$self->listmain(
			'template' => $temp,
			'block'    => 'sb_status_entusr_list',
			'objects'  => \@entry,
			'tags'     => {
				'sb_entusr_subj' => sub { (shift)->display_entry_title(@_) },
				'sb_entusr_date' => 'date',
				'sb_entusr_stat' => sub { (shift)->display_entry_status(@_) },
			},
		);
	}
	{ # --- comments
		$temp->num(0);
		$temp->tag('sb_com_close'=>sb::Data->count('Message','cond'=>{'stat'=>0}));
		my @message = sb::Data->load('Message',
			'sort'  => 'date',
			'order' => 1,
			'num'   => STATUS_NUM,
		);
		$temp->tag('sb_com_num'=>sb::Data->matched);
		$self->listmain(
			'template' => $temp,
			'block'    => 'sb_status_com_list',
			'objects'  => \@message,
			'tags'     => {
				'sb_com_author' => sub { (shift)->display_message_author(@_) },
				'sb_com_date'   => 'date',
				'sb_com_stat'   => sub { (shift)->display_message_status(@_) },
			},
		);
	}
	{ # --- trackback
		$temp->num(0);
		$temp->tag('sb_tb_close'=>sb::Data->count('Trackback','cond'=>{'stat'=>0}));
		my @trackback = sb::Data->load('Trackback',
			'sort'  => 'date',
			'order' => 1,
			'num'   => STATUS_NUM,
		);
		$temp->tag('sb_tb_num'=>sb::Data->matched);
		$self->listmain(
			'template' => $temp,
			'block'    => 'sb_status_tb_list',
			'objects'  => \@trackback,
			'tags'     => {
				'sb_tb_name' => sub { (shift)->display_message_author(@_) },
				'sb_tb_date' => 'date',
				'sb_tb_stat' => sub { (shift)->display_message_status(@_) },
			},
		);
	}
}
sub _display_appearance
{
	my $self = shift;
	my $temp = shift;
	$temp->num(0);
	$temp->tag('sb_blog_title'=>$self->{'blog'}->title);
	$temp->tag('sb_blog_desc'=>$self->{'blog'}->text);
	foreach my $key (@{$mConfigItems{'disp'}})
	{
		$temp->tag('sb_' . $key => $self->conf->value('conf_' . $key));
	}
	foreach my $key (@{$mConfigItems{'sort'}})
	{
		$temp->tag('sb_' . $key => 'selected="selected"') if ($self->conf->value('conf_' . $key));
	}
}
sub _display_basic
{
	my $self = shift;
	my $temp = shift;
	my $selector = '';
	$temp->num(0);
	foreach my $mode ( @{$self->conf->value('setup_entry_archive')} )
	{
		my $label = sb->string('entryarchive_' . $mode);
		$label .= ' (' . $mode . ')';
		$selector .= '<option value="' . $mode . '"';
		$selector .= ' selected="selected"' if ($mode eq $self->conf->value('conf_entry_archive'));
		$selector .= '>' . $label . '</option>';
	}
	$temp->tag('sb_entry_archive'=>$selector);
	if ( eval('require Image::Magick') )
	{
		$temp->num(0);
		$temp->tag('sb_thumbsize'=>$self->conf->value('conf_thumbsize'));
		$temp->tag('sb_thumbcheck'=>'selected="selected"') if ($self->conf->value('conf_thumbcheck'));
		$temp->block('sb_config_thumb'=>1);
	}
	$temp->tag('sb_imagename'=>'selected="selected"') if ($self->conf->value('conf_imagename'));
	foreach my $key (@{$mConfigItems{'base'}})
	{
		$temp->tag('sb_' . $key => $self->conf->value('conf_' . $key));
	}
	$self->timezone_selector(
		'temp'    => $temp,
		'tag'     => 'sb_config_zone',
		'current' => $self->conf->value('conf_timezone'),
	);
	$temp->tag('sb_sendmail_' . $self->{'blog'}->stype =>'selected="selected"') if ($self->{'blog'}->stype ne '');
	$temp->tag('sb_smtp'=>sb::Text->new($self->{'blog'}->smtp)->entitized());
	$temp->block('sb_config_smtp'=>1) if ( eval('require Net::SMTP;') );
	if ( eval('require Net::POP3;') )
	{
		$temp->num(0);
		$temp->tag('sb_popserver'   =>sb::Text->new($self->{'blog'}->psrv)->entitized());
		$temp->tag('sb_popacount'   =>sb::Text->new($self->{'blog'}->pacc)->entitized());
		$temp->tag('sb_poppassword' =>sb::Text->new($self->{'blog'}->ppass)->entitized());
		$temp->tag('sb_popsubject'  =>sb::Text->new($self->{'blog'}->psubj)->entitized());
		$temp->block('sb_config_pop'=>1);
	}
	my @langs = $self->_get_available_languages();
	if ( $#langs > 0 )
	{
		my $lang_sel = '';
		foreach my $lang ( @langs )
		{
			my $module = 'sb::Language::' . $lang;
			my $name = $module->name();
			$name = sb->string($name) . ' (' . $name . ')' if ( sb->string($name) ne $name );
			$lang_sel .= '<option value="' . $lang . '"';
			$lang_sel .= ' selected="selected"' if ( $lang eq $self->lang->language );
			$lang_sel .= '>' . $name . '</option>';
		}
		$temp->num(0);
		$temp->tag('sb_lang'=>$lang_sel);
		$temp->block('sb_config_lang_select'=>1);
	}
}
sub _display_plugin
{
	my $self = shift;
	my $temp = shift;
	my $plugin = sb->get->plugin;
	my $num = 0;
	my %details_of = %{ $plugin->plugin() };
	foreach my $name ( @{$plugin->lists()} )
	{
		my $details = ( $details_of{$name} ) 
			? $details_of{$name} 
			: $plugin->read_plugin($name);
		next if (!$details);
		my $author = ($details->{'author'} ne '') ? $details->{'author'} : PLUGIN_NONAME;
		if ($details->{'detail'} =~ /s?https?:\/\/[-_.!~*\'()a-zA-Z0-9;\/?:\@&=+\$,%#]+/)
		{
			$author = sprintf(PLUGIN_LINK,$details->{'detail'},$author);
		}
		my $plugin_name = $details->{'name'};
		if ($details->{'text'} ne '')
		{
			$plugin_name = sprintf(PLUGIN_DETAIL,$details->{'text'},$plugin_name);
		}
		if ( $details->{'type'} ne '' )
		{
			$details->{'type'} = sprintf(PLUGIN_DETAIL,$details->{'type'},$details->{'type'});
		}
		$temp->num($num);
		$temp->tag('sb_plugin_file'=>$name);
		$temp->tag('sb_plugin_version'=>$details->{'version'});
		$temp->tag('sb_plugin_name'=>$plugin_name);
		$temp->tag('sb_plugin_auth'=>$author);
		$temp->tag('sb_plugin_type'=>$details->{'type'});
		$temp->tag('sb_plugin_flag'=>($plugin->ignore($name)) ? '' : 'checked="checked"');
		$temp->tag('sb_list_class'=>($num % 2) ? 'odd' : 'even');
		$num++;
	}
	$temp->num(0);
	$temp->tag('sb_plugin_debug'=>$plugin->set());
	if ($num > 0)
	{
		$temp->block('sb_config_plugin_list'=>$num);
		$temp->block('sb_config_plugin_select'=>1);
	}
	else
	{
		$temp->block('sb_config_noplugin'=>1);
	}
}
sub _get_message
{
	my $self = shift;
	my $msg = '';
	$msg .= sb->string('Settings have been modified.') ."\n";
	$msg .= sb->string('You need to rebuild your weblog to reflect the changes to the previous articles.');
	$msg .= sprintf(sb->string('&#8658; <a href="%s">Rebuild site</a>'),$self->script_path . '?__mode=build') . "\n";
	return $msg;
}
sub _get_available_languages
{
	my $self = shift;
	my $dir = $self->conf->value('dir_temp');
	opendir(TEMP_DIR,$dir);
	my @checks = grep { s/^(\w+?)\.txt$/$1/ } readdir(TEMP_DIR);
	closedir(TEMP_DIR);
	my @langs;
	foreach my $check ( @checks )
	{
		next if ( !-e $dir . $check );
		next if ( !-d $dir . $check );
		next if ( !-r $dir . $check . '/' . sb::App::Admin::TEMPLATE() );
		my $module = 'sb::Language::' . $check;
		next if ( !sb::load_module( $module ) );
		next if ( $module->name() eq '' );
		push( @langs, $check );
	}
	return @langs;
}
1;
__END__
